@@ -4,7 +4,7 @@
 # category: ui
 # title: Channel plugins
 # description: Base implementation for channels and feature plugins
-# version: 1.6
+# version: 1.7
 # license: public domain
 # author: mario
 # url: http://fossil.include-once.org/streamtuner2/
@@ -38,15 +38,65 @@ import inspect
 
 # Only export plugin classes and a few utility functions
 __all__ = [
-    "GenericChannel", "ChannelPlugin", "use_rx", "mime_fmt",
-    "entity_decode", "strip_tags", "nl", "unhtml", "to_int"
+    "FeaturePlugin", "GenericChannel", "ChannelPlugin", "use_rx", "mime_fmt",
+    "stub_parent", "entity_decode", "strip_tags", "nl", "unhtml", "to_int"
 ]
 __path__.insert(0, conf.dir + "/plugins")
 
 
 
+# Base class for plugins (features or channels)
+class FeaturePlugin(object):
+
+    # plugin meta and object references
+    module = ""
+    meta = { "config": [] }
+    parent = None     # main window
+
+    # minimum setup for ChannelPlugins and feature hooks
+    def __init__(self, parent, *k, **kw):
+
+        # set up meta infos
+        self.parent = parent
+        self.module = self.__class__.__name__
+        self.meta = plugin_meta(src = inspect.getcomments(inspect.getmodule(self)))
+        self.config = self.meta.get("config", [])
+        self.title = self.meta.get("title", self.module)
+
+        # add default options values to config.conf.* dict
+        conf.add_plugin_defaults(self.meta, self.module)
+
+        # implicit action handler registration
+        if hasattr(self, "resolve_urn"):
+            action.handler["urn:%s" % self.module] = self.resolve_urn
+
+        # secondary init function
+        self.init2(parent)
+
+    # optionally to be overriden by plugins (run after base __init__)
+    def init2(self, parent, *k, **kw):
+        pass
+
+    # Statusbar stub (defers to parent/main window, if in GUI mode)
+    def status(self, *args, **kw):
+        if self.parent:
+            self.parent.status(*args, **kw)
+        else:
+            log.INFO("status():", *v)
+        
+    # Statusbar with highlighting and default icon
+    def warn(self, text, *args, **kw):
+        if isinstance(text, (str, unicode)) and "status_color" in conf:
+            text = "<span background='%s'>%s</span>" % (conf.status_color, text)
+            kw["markup"] = 1
+        if not "icon" in kw:
+            kw["icon"] = "gtk-dialog-warning"
+        self.status(text, *args, **kw)
+
+
+
 # Generic channel module
-class GenericChannel(object):
+class GenericChannel(FeaturePlugin):
 
     # control attributes
     meta = { "config": [] }
@@ -71,7 +121,6 @@ class GenericChannel(object):
     pix_entry = None  # ListStore entry that contains favicon
     img_resize = None  # Rescale `img` references to icon size
     fixed_size = [24,24]  # Default height+width for favicons
-    parent = None     # reference to main window
 
     # mapping of stream{} data into gtk treeview/treestore representation
     datamap = [
@@ -128,19 +177,9 @@ class GenericChannel(object):
         #self.streams = {}
         self.gtk_list = None
         self.gtk_cat = None
-        self.module = self.__class__.__name__
-        self.meta = plugin_meta(src = inspect.getcomments(inspect.getmodule(self)))
-        self.config = self.meta.get("config", [])
-        self.title = self.meta.get("title", self.module)
-
-        # add default options values to config.conf.* dict
-        conf.add_plugin_defaults(self.meta, self.module)
         
-        # extra init function
-        if hasattr(self, "init2"):
-            self.init2(parent)
-        if hasattr(self, "resolve_urn"):
-            action.handler["urn:%s" % self.module] = self.resolve_urn
+        # base init (meta infos, parent reference, init2)
+        FeaturePlugin.__init__(self, parent)
         
         # Only if streamtuner2 is run in graphical mode        
         if (parent):
@@ -195,12 +234,6 @@ class GenericChannel(object):
         # no longer using `conf.show_favicons`
 
 
-    # Statusbar stub (defers to parent/main window, if in GUI mode)
-    def status(self, *args, **kw):
-        if self.parent: self.parent.status(*args, **kw)
-        else: log.INFO("status():", *v)
-
-
         
     #--------------------- streams/model data accesss ---------------------------
 
@@ -255,7 +288,7 @@ class GenericChannel(object):
     def row(self):
         row = self.stations() [ self.rowno() ]
         # resolve stream url for some plugins
-        if row.get("url", "urn:x-streamtuner2:no").startswith("urn:"):
+        if row.get("url") and row.get("url", "urn:x-streamtuner2:no").startswith("urn:"):
             self.status("Resolving actual stream URL for `%s`" % row["url"], timeout=2)
             # most urn: handlers update `row` - some return a new value - which is handled in action.run_fmt_url() however
             action.resolve_urn(row)  # row = ..() or row
@@ -571,6 +604,8 @@ class GenericChannel(object):
             listformat = row.get("listformat", self.listformat)
             # invoke audio player
             action.play(row, audioformat, listformat)
+        elif row.get("homepage") and row.get("format") == "text/html":
+            action.browser(row["homepage"])
         else:
             self.status("No station selected for playing.")
         return row
@@ -588,15 +623,6 @@ class GenericChannel(object):
 
     
 
-        
-    
-
-
-
-
-
-
-
 
 # channel plugin without glade-pre-defined notebook tab
 #
@@ -683,6 +709,8 @@ class ChannelPlugin(GenericChannel):
 
 
 
+
+
 # WORKAROUND for direct channel module imports,
 # eases instantiations without GUI a little,
 # reducing module dependencies (conf. / ahttp. / channels. / parent.) would be better
